from fastapi import HTTPException
from fastapi.datastructures import QueryParams
from sqlmodel import select

from openec2.config import OpenEC2Config
from openec2.db import DatabaseDep
from openec2.db.image import AMI
from openec2.images import garbage_collect_image

def deregister_image(
    params: QueryParams,
    config: OpenEC2Config,
    db: DatabaseDep,
):
    image_id = params["ImageId"]
    ami = db.exec(select(AMI).where(AMI.id == image_id)).one()
    if ami is None:
        raise HTTPException(status_code=404, detail="Unknown AMI")

    # Mark the image as deregistered
    ami.deregistered = True
    db.add(ami)
    db.commit()

    # First round of garbage collection
    garbage_collect_image(image_id, db)
