from typing import cast
from pathlib import Path
from urllib.parse import urlparse
import uuid
import shutil

from fastapi.datastructures import QueryParams
import requests

from openec2.config import OpenEC2Config
from openec2.db import DatabaseDep
from openec2.db.image import AMI

def import_image(
    params: QueryParams,
    config: OpenEC2Config,
    db: DatabaseDep,
):
    first_disk_image_url = params["DiskContainer.1.Url"]
    url = urlparse(first_disk_image_url)
    ami_id = uuid.uuid4().hex

    imageLocation = cast(Path, config.images)
    imageLocation.mkdir(exist_ok=True)
    dst = imageLocation / ami_id
    filename = Path(url.path).name
    match url.scheme:
        case "https":
            with dst.open("wb") as f:
                with requests.get(first_disk_image_url, stream=True) as r:
                    r.raise_for_status()
                    for chunk in r.iter_content(8196):
                        f.write(chunk)
        case "file":
            shutil.copy(
                url.path,
                str(dst),
            )
        case _:
            raise Exception("Unknown scheme")

    db.add(
        AMI(
            id=ami_id,
            description=None,
            originalFilename=filename,
        )
    )
    db.commit()
    return ami_id
