import uuid

from fastapi import HTTPException
from fastapi.datastructures import QueryParams
from sqlmodel import select

from openec2.libvirt import LibvirtSingleton
from openec2.config import OpenEC2Config
from openec2.db import DatabaseDep
from openec2.db.instance import Instance
from openec2.utils.array import parse_array_plain
from openec2.api.shared import InstanceState
from openec2.api.stop_instances import StopInstancesResponse, StopInstancesResponseInstancesSet, StopInstancesResponseInstancesSetInstance


def stop_instances(
    params: QueryParams,
    config: OpenEC2Config,
    db: DatabaseDep,
):
    conn = LibvirtSingleton.of().connection
    instances: list[StopInstancesResponseInstancesSetInstance] = []
    for instance_id in parse_array_plain("InstanceId", params):
        instance = db.exec(select(Instance).where(Instance.id == instance_id)).first()
        if instance is None:
            raise HTTPException(status_code=404, detail="Unknown instance")

        dom = conn.lookupByName(instance_id)
        running = dom.isActive()
        prev_state = InstanceState(
            code=16 if running else 80,
            name="running" if running else "stopped",
        )
        if running:
            dom.shutdown()
        running = dom.isActive()
        current_state = InstanceState(
            code=16 if running else 80,
            name="running" if running else "stopped",
        )

        instances.append(
            StopInstancesResponseInstancesSetInstance(
                instance_id=instance_id,
                previous_state=prev_state,
                current_state=current_state,
            ),
        )

    return StopInstancesResponse(
        request_id=uuid.uuid4().hex,
        instances_set=StopInstancesResponseInstancesSet(
            item=instances,
        ),
    ).to_xml()
