from pydantic_xml import BaseXmlModel, element
import libvirt

from openec2.db.instance import Instance
from openec2.api.shared import InstanceState


class InstanceTag(BaseXmlModel):
    key: str = element()
    value: str = element()

class InstanceTagSet(BaseXmlModel):
    item: list[InstanceTag] = element()

class InstanceDescription(
    BaseXmlModel,
    tag="item",
):
    instance_id: str = element(tag="instanceId")
    image_id: str = element(tag="imageId")
    instance_state: InstanceState = element(tag="instanceState")
    tag_set: InstanceTagSet = element(tag="tagSet")

class DescribeInstancesResponseReservationSet(
    BaseXmlModel,
    tag="reservationSet",
):
    item: list[InstanceDescription] = element()

class DescribeInstancesResponse(
    BaseXmlModel,
    tag="DescribeInstancesResponse",
    nsmap={"": "http://ec2.amazonaws.com/doc/2016-11-15/"}
):
    request_id: str = element(tag="requestId")
    reservation_set: DescribeInstancesResponseReservationSet = element("reservationSet")


def describe_instance(instance: Instance, domain: libvirt.virDomain) -> InstanceDescription:
    running = domain.isActive()
    return InstanceDescription(
        instance_id=instance.id,
        image_id=instance.imageId,
        instance_state=InstanceState(
            code=16 if running else 80,
            name="running" if running else "stopped",
        ),
        tag_set=InstanceTagSet(
            item=[
                InstanceTag(
                    key=key,
                    value=value,
                ) for key, value in instance.tags.items()
            ],
        ),
    )
