from pathlib import Path
from fastapi import Depends
from pydantic import BaseModel
from typing import Annotated

class _OpenEC2InstanceType(BaseModel):
    memory: int # MiB
    vcpu: float
    disk: int # Gi

class _OpenEC2InstanceConfig(BaseModel):
    location: Path

    types: dict[str, _OpenEC2InstanceType]

class _OpenEC2LibvirtConfig(BaseModel):
    connection: str

class _OpenEC2Config(BaseModel):
    images: Path
    seed: Path
    instances: _OpenEC2InstanceConfig
    libvirt: _OpenEC2LibvirtConfig

def _get_config() -> _OpenEC2Config:
    # TODO: Read from disk
    return _OpenEC2Config(
        images=Path("/home/alexander/openec2/images"),
        seed=Path("/home/alexander/openec2/seed"),
        instances=_OpenEC2InstanceConfig(
            location=Path("/home/alexander/openec2/instances"),
            types={
                "micro": _OpenEC2InstanceType(
                    memory=1024,
                    vcpu=1,
                    disk=10,
                ),
            },
        ),
        libvirt=_OpenEC2LibvirtConfig(
            connection="qemu:///system"
        ),
    )

class ConfigSingleton:
    __instance: "ConfigSingleton | None" = None

    config: _OpenEC2Config

    def __init__(self):
        self.config = _get_config()

    def get_config(self) -> _OpenEC2Config:
        return self.config

    @staticmethod
    def of() -> "ConfigSingleton":
        if ConfigSingleton.__instance is None:
            ConfigSingleton.__instance = ConfigSingleton()
        return ConfigSingleton.__instance

OpenEC2Config = Annotated[_OpenEC2Config, Depends(ConfigSingleton.of().get_config)]
